package com.ruoyi.market.domain;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.handlers.FastjsonTypeHandler;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Size;
import java.time.LocalDate;
import java.time.LocalDateTime;


/**
 * 奖品库对象 market_award_library
 */

@Data
@TableName(value = "market_award_library",autoResultMap = true)
public class MarketAwardLibrary {

    private String id;

    @ApiModelProperty("奖品名称")
    @NotBlank(message = "请填写奖品名称")
    @Size(max=50,message = "奖品名称不能超过50个字符")
    private String awardName;

    @ApiModelProperty("奖品数量")
    @NotBlank(message = "请填写奖品数量")
    @Min(value = 1,message = "奖品数量不能小于1")
    @Max(value = 999999, message = "奖品数量不能超过999999")
    private String awardNum;

    @ApiModelProperty("奖品图片")
    @NotBlank(message = "请上传奖品图片")
    private String awardPhotoUrl;

    @ApiModelProperty("奖品类型")
    @NotBlank(message = "请选择奖品类型")
    private String awardType;

    @ApiModelProperty("奖品属性")
    @TableField(value = "award_attr",typeHandler = FastjsonTypeHandler.class)
    private JSONObject awardAttr;

//    @ApiModelProperty("奖品属性对象")
//    private JSONObject awardAttrObj;

    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @ApiModelProperty("生效日期")
    private LocalDateTime effectiveDate;

    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @ApiModelProperty("失效日期")
    private LocalDateTime expiryDate;

    @ApiModelProperty("是否删除")
    private String isDelete;

    @ApiModelProperty("奖品位置")
    @TableField(exist = false)
    private Integer pos;

    @ApiModelProperty("奖品类型名称")
    @TableField(exist = false)
    private String typeName;
}
