package com.ruoyi.market.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.market.domain.*;
import com.ruoyi.market.domain.vo.AdvertisingLibraryQueryVo;
import com.ruoyi.market.domain.vo.AdvertisingVo;
import com.ruoyi.market.domain.vo.AdvertisingWatchCount;
import com.ruoyi.market.mapper.MarketAdvertisingLibraryMapper;
import com.ruoyi.market.mapper.MarketAdvertisingLogMapper;
import com.ruoyi.market.service.*;
import com.ruoyi.common.utils.idgen.IdGenerate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static com.ruoyi.market.constants.AdvertisingConstants.REDIS_ADVERTISING_KEY;


@Service
public class MarketAdvertisingLibraryServiceImpl extends ServiceImpl<MarketAdvertisingLibraryMapper, MarketAdvertisingLibrary>
        implements MarketAdvertisingLibraryService {

    @Resource
    private MarketAdvertisingCompanyService marketAdvertisingCompanyService;

    @Resource
    private MarketAdvertisingLibraryService marketAdvertisingLibraryService;

    @Resource
    private MarketAdvertisingLogService marketAdvertisingLogService;

    @Resource
    private MarketAdvertisingConfigService marketAdvertisingConfigService;

    @Resource
    private MarketAdvertisingAwardService marketAdvertisingAwardService;

    @Resource
    private MarketAwardLogService marketAwardLogService;

    @Resource
    private MarketAdvertisingLogMapper marketAdvertisingLogMapper;

    @Resource
    RedisCache redisCache;


    @Override
    @Transactional
    public boolean insertAdvertisingLibrary(MarketAdvertisingLibrary advertisingLibrary) {
        advertisingLibrary.setId(IdGenerate.nextId());
        String[] awardIds = advertisingLibrary.getAwardIds();
        String awardIdsString = String.join(",", awardIds);
        advertisingLibrary.setAwardId(awardIdsString);
        this.save(advertisingLibrary);
        saveMarketAdvertisingAwards(advertisingLibrary, awardIds);
        return true;
    }

    @Override
    @Transactional
    public boolean updateAdvertisingLibrary(MarketAdvertisingLibrary advertisingLibrary) {
        String[] awardIds = advertisingLibrary.getAwardIds();
        String awardIdsString = String.join(",", awardIds);
        advertisingLibrary.setAwardId(awardIdsString);
        marketAdvertisingAwardService.remove
                (new LambdaQueryWrapper<MarketAdvertisingAward>().eq(MarketAdvertisingAward::getVideoId, advertisingLibrary.getId()));
        this.save(advertisingLibrary);
        saveMarketAdvertisingAwards(advertisingLibrary, awardIds);
        return this.updateById(advertisingLibrary);
    }

    private boolean saveMarketAdvertisingAwards(MarketAdvertisingLibrary advertisingLibrary, String[] awardIds) {
        List<MarketAdvertisingAward> awards = new ArrayList<>();

        for (String awardId : awardIds) {
            MarketAdvertisingAward marketAdvertisingAward = new MarketAdvertisingAward();
            marketAdvertisingAward.setId(IdGenerate.nextId());
            marketAdvertisingAward.setAwardId(awardId);
            marketAdvertisingAward.setVideoId(advertisingLibrary.getId());
            marketAdvertisingAward.setCreateTime(LocalDateTime.now());
            awards.add(marketAdvertisingAward);
        }

        marketAdvertisingAwardService.saveBatch(awards);
        return true;
    }

    @Override
    public boolean deleteAdvertisingLibrary(MarketAdvertisingLibrary advertisingLibrary) {
        return this.removeById(advertisingLibrary);
    }

    @Override
    public Page<MarketAdvertisingLibrary> advertisingLibraryList(Page page,AdvertisingLibraryQueryVo advertisingLibraryQueryVo) {
        return baseMapper.selectAdvertisingLibraryList(page, advertisingLibraryQueryVo);
    }

    @Override
    public MarketAdvertisingLibrary getCompanyAdvertising(AdvertisingVo advertisingVo) {
        // 获取本用户今天观看广告列表 已看完次数和最后一次观看完成时间
        LocalDateTime startOfDay = LocalDateTime.of(LocalDate.now(), LocalTime.MIN);
        LocalDateTime endOfDay = LocalDateTime.of(LocalDate.now(), LocalTime.MAX);
        LambdaQueryWrapper<MarketAdvertisingLog> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(MarketAdvertisingLog::getUserId, advertisingVo.getUserId())
                .between(MarketAdvertisingLog::getCreateTime, startOfDay, endOfDay)
                .eq(MarketAdvertisingLog::getIsCatOver, 1)
                .orderByDesc(MarketAdvertisingLog::getCreateTime);
        List<MarketAdvertisingLog> advertisingLogList = marketAdvertisingLogService.list(queryWrapper);
        MarketAdvertisingConfig advertisingConfig = marketAdvertisingConfigService.list().get(0);
        if (advertisingLogList.size() == advertisingConfig.getDailyAdLimit()) {
            throw new RuntimeException("今日观看广告次数已达上限");
        }
        if (!advertisingLogList.isEmpty()) {
            // 如果今天已经观看过广告，则判断是否距离上次观看时间足够

            // 如果距离上次观看时间过短，则提示用户不能观看广告
            LocalDateTime lastCompletedTime = advertisingLogList.get(0).getCreateTime(); //获取上次观看广告的时间
            LocalDateTime currentTime = LocalDateTime.now(); //获取当前时间
            Duration durationSinceLastAd = Duration.between(lastCompletedTime, currentTime); //计算距离上次观看广告的时间
            long secondsSinceLastAd = durationSinceLastAd.getSeconds(); //转换为秒

            if (secondsSinceLastAd < advertisingConfig.getAdInterval()) {
                throw new RuntimeException("距离上次观看广告时间过短，暂时不能观看");
            }
        }


        // 按公司分配权重抽取播放广告的公司
        List<MarketAdvertisingCompany> advertisingCompanyList = marketAdvertisingCompanyService.list();
        BigDecimal totalProbability = BigDecimal.ZERO;

        for (MarketAdvertisingCompany advertisingCompany : advertisingCompanyList) {
            totalProbability = totalProbability.add(advertisingCompany.getAdvertisingProbability());
        }
        Random random = new Random();
        double randomProbability = random.nextDouble() * 10;

        BigDecimal cumulativeProbability = BigDecimal.ZERO;
        for (MarketAdvertisingCompany company : advertisingCompanyList) {
            cumulativeProbability = cumulativeProbability.add(company.getAdvertisingProbability());
            if (cumulativeProbability.compareTo(BigDecimal.valueOf(randomProbability)) >= 0) {
                String companyId = company.getId();
                List<MarketAdvertisingLibrary> filteredAds = marketAdvertisingLibraryService.list().stream()
                        .filter(ad -> ad.getVideoCompany().equals(companyId))
                        .collect(Collectors.toList());
                if (filteredAds.isEmpty()) {
                    throw new RuntimeException("没有找到广告");
                }
                Random randomAd = new Random();
                int randomIndex = randomAd.nextInt(filteredAds.size());
                redisCache.setCacheObject(REDIS_ADVERTISING_KEY + advertisingVo.getUserId(), filteredAds.get(randomIndex).getId(),60 , TimeUnit.SECONDS);
                //抽奖日志
                String awardIdArray = filteredAds.get(randomIndex).getAwardId();
                String[] awardIds = awardIdArray.split(",");
                for (String awardId : awardIds) {
                    //添加发放日志记录
                    MarketAwardLog marketAwardLog = new MarketAwardLog("",awardId,advertisingVo.getUserId(),"1","1");
                    marketAwardLogService.save(marketAwardLog);
                }
                return filteredAds.get(randomIndex);
            }
        }

        // 理论上如果权重分配正确到不了这一步
        return null;
    }


    //当天观看广告次数/全天观看广告次数
    public AjaxResult getAdvertisingWatchCount(AdvertisingWatchCount advertisingWatchCount){
        //获取每日广告观看限制次数
        MarketAdvertisingConfig advertisingConfig = marketAdvertisingConfigService.list().get(0);

        advertisingWatchCount.setTotal(String.valueOf(advertisingConfig.getDailyAdLimit()));
        advertisingWatchCount.setUserId(advertisingWatchCount.getUserId());
        //当天时间
        LocalDateTime startOfDay = LocalDateTime.of(LocalDate.now(), LocalTime.MIN);
        LocalDateTime endOfDay = LocalDateTime.of(LocalDate.now(), LocalTime.MAX);
        LambdaQueryWrapper<MarketAdvertisingLog> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(MarketAdvertisingLog::getUserId, advertisingWatchCount.getUserId())
                .eq(MarketAdvertisingLog::getIsCatOver, 1)
                .between(MarketAdvertisingLog::getCreateTime, startOfDay, endOfDay);
        //获取当天观看次数
        advertisingWatchCount.setWatchCount(String.valueOf(marketAdvertisingLogService.count(queryWrapper)));
        //获取当天观看次数列表
        advertisingWatchCount.setMarketAdvertisingLogList(marketAdvertisingLogService.list(queryWrapper));
        return AjaxResult.success(advertisingWatchCount);
    }


    //统计今日全部用户观看次数
    public Page<AdvertisingWatchCount> getAllUserAdvertisingWatchCount(Page page){
        return marketAdvertisingLogMapper.selectAllMarketAdvertisingLog(page);
    }
}
