package com.ruoyi.market.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.market.domain.*;
import com.ruoyi.market.domain.vo.AwardQueryVo;
import com.ruoyi.market.mapper.MarketAwardLibraryMapper;
import com.ruoyi.market.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Service
public class MarketAwardLibraryServiceImpl extends ServiceImpl<MarketAwardLibraryMapper, MarketAwardLibrary>
        implements MarketAwardLibraryService {

    @Autowired
    MarketUserDrawService marketUserDrawService;

    @Autowired
    MarketDrawActivityProbabilityService marketDrawActivityProbabilityService;


    @Autowired
    MarketDrawActivityService marketDrawActivityService;

    @Autowired
    MarketAwardLogService marketAwardLogService;

    @Autowired
    MarketAdvertisingAwardService marketAdvertisingAwardService;

    @Autowired
    MarketAwardLibraryService marketAwardLibraryService;

    @Resource
    WechatPayService wechatPayService;

    @Resource
    MarketUserWechatService marketUserWechatService;

    /**
     * 查询奖品列表
     *
     * @param page
     * @param awardQueryVo
     * @return
     */
    @Override
    public Page<MarketAwardLibrary> awardLibraryList(Page page, AwardQueryVo awardQueryVo) {
        Page<MarketAwardLibrary> awardLibraryList = baseMapper.getAwardLibraryList(page, awardQueryVo);
        return awardLibraryList;
    }

    /**
     * 删除奖品
     *
     * @param awardId
     * @return
     */
    @Override
    public boolean delAward(String awardId) {
        MarketAwardLibrary award = this.getById(awardId);
        if (Objects.isNull(award)) {
            throw new RuntimeException("奖品不存在");
        }
        award.setIsDelete("1");
        this.updateById(award);
        return true;
    }

    /**
     * 发放奖品
     * @param activityId    活动id
     * @param userId        用户id
     * @return
     */
    @Override
    @Transactional
    public boolean grantAward(String activityId, String userId,Integer pos) {
        //确认奖品的类型
        List<MarketDrawActivityProbability> probabilitysByDrawActivityId = marketDrawActivityProbabilityService.getProbabilitysByDrawActivityId(activityId);
        MarketDrawActivityProbability marketDrawActivityProbability = probabilitysByDrawActivityId.stream().filter(x -> Objects.equals(x.getPos(), pos)).findFirst().orElse(null);
        if(Objects.isNull(marketDrawActivityProbability)){
            throw new RuntimeException("没有找到奖品");
        }
        MarketAwardLibrary award = this.getById(marketDrawActivityProbability.getAwardId());
        if(award.getId().equals("1815679374784438275")) {
//            JSONObject obj = JSONObject.parse(award.getAwardAttr());
            String drawActivityId = award.getAwardAttr().getString("activityId");
            String number = award.getAwardAttr().getString("number");
            //发放抽奖奖品
            this.numberDraws(drawActivityId,userId,Integer.parseInt(number));
            MarketAwardLog marketAwardLog = new MarketAwardLog(activityId,award.getId(),userId,"0","0");
            marketAwardLogService.save(marketAwardLog);
        }
        if (award.getId().equals("1827970570895597568")) {
            MarketUserWechat marketUserWechat = marketUserWechatService.getById(userId);
            String amount = award.getAwardAttr().getString("amount");
            wechatPayService.transfer(marketUserWechat.getOpenId(), Long.valueOf(amount));
            MarketAwardLog marketAwardLog = new MarketAwardLog(activityId,award.getId(),userId,"0","1");
            marketAwardLogService.save(marketAwardLog);
        } else {
            //添加发放日志记录
            MarketAwardLog marketAwardLog = new MarketAwardLog(activityId,award.getId(),userId,"0","1");
            marketAwardLogService.save(marketAwardLog);
        }
        return true;
    }

    /**
     * 抽奖次数
     * @param activityId
     * @param userId
     * @return
     */
    private boolean numberDraws(String activityId, String userId, Integer number) {
        try {
            MarketUserDraw userDrawNumber = marketUserDrawService.getUserDrawNumber(userId, activityId);
            //添加抽奖次数
            userDrawNumber.setAwardNumber(userDrawNumber.getAwardNumber() + number);
            marketUserDrawService.updateById(userDrawNumber);
            return true;
        }catch(Exception err){
            return false;
        }
    }

    /**
     * 红包
     * @return
     */
    private boolean redEnvelope(String awardId,String userId,Integer money){
        try{

            return true;
        }catch(Exception err){
            return false;
        }
    }


    //广告奖励发放
    public boolean adsGrantAward(String adId,String userId){
        List<MarketAdvertisingAward> marketAdvertisingAwards = marketAdvertisingAwardService.getListByAdvertisingId(adId);
        List<String> awardIds = new ArrayList<>();
        marketAdvertisingAwards.forEach(x->{
            awardIds.add(x.getAwardId());
        });
        List<MarketAwardLibrary> getAward = marketAwardLibraryService.listByIds(awardIds);
        getAward.forEach(x->{
            switch(x.getAwardType()){
                case "1817870418863120386":
//                this.redEnvelope(activityId,userId,);
                    break;
                case "1818542508308500481":
//                    JSONObject obj = JSONObject.parse(x.getAwardAttr());
                    String drawActivityId = x.getAwardAttr().getString("activityId");
                    String number = x.getAwardAttr().getString("number");
                    //发放抽奖奖品
                    this.numberDraws(drawActivityId,userId,Integer.parseInt(number));
                    break;
            }
            MarketAwardLog marketAwardLog = new MarketAwardLog(adId,x.getId(),userId,"1","1");
            marketAwardLogService.save(marketAwardLog);
        });
        return true;
    }


}
