package com.ruoyi.web.controller.api;


import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.market.domain.MarketDrawActivity;
import com.ruoyi.market.domain.MarketDrawActivityProbability;
import com.ruoyi.market.domain.MarketUserDraw;
import com.ruoyi.market.domain.vo.AwardInfoVo;
import com.ruoyi.market.domain.vo.DrawActivityInfoVo;
import com.ruoyi.market.domain.vo.DrawVo;
import com.ruoyi.market.service.MarketAwardLogService;
import com.ruoyi.market.service.MarketDrawActivityService;
import com.ruoyi.market.service.MarketUserDrawService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;
import java.util.concurrent.TimeUnit;


import static com.ruoyi.market.constants.DrawConstants.REDIS_DRAW_ACTIVITY_KEY;

@Api(tags = "前端抽奖api管理")
@RestController
@RequestMapping("/draw")
public class DrawApiController {

    @Autowired
    MarketDrawActivityService marketDrawActivityService;

    @Autowired
    MarketUserDrawService marketUserDrawService;

    @Autowired
    MarketAwardLogService marketAwardLogService;


    @Autowired
    RedisCache redisCache;

    @ApiOperation("获取抽奖参数")
    @PostMapping("getDrawParams")
    public AjaxResult getDrawParams(@RequestBody DrawVo drawVo) {
        List<AwardInfoVo> awardInfoVos = marketDrawActivityService.getForstDrawActivity(drawVo.getDrawActivityId());
        MarketDrawActivity drawActivity = marketDrawActivityService.getById(drawVo.getDrawActivityId());
        MarketUserDraw userDrawNumber = marketUserDrawService.getUserDrawNumber(drawVo.getUserId(), drawVo.getDrawActivityId());
        if(Objects.isNull(userDrawNumber)){
            userDrawNumber = new MarketUserDraw();
            userDrawNumber.setActivityId(drawVo.getDrawActivityId());
            userDrawNumber.setUserId(drawVo.getUserId());
            userDrawNumber.setAwardNumber(drawActivity.getDefaultDrawNum());
            marketUserDrawService.save(userDrawNumber);
        }
        DrawActivityInfoVo drawActivityInfoVo = new DrawActivityInfoVo(drawActivity,awardInfoVos,userDrawNumber.getAwardNumber());
        return AjaxResult.success(drawActivityInfoVo);
    }

    @ApiOperation("发起抽奖")
    @PostMapping("startDraw")
    public AjaxResult startDraw(@RequestBody DrawVo drawVo) {
        Object cacheObject = redisCache.getCacheObject(REDIS_DRAW_ACTIVITY_KEY +  drawVo.getDrawActivityId() + drawVo.getUserId());
        if(!Objects.isNull(cacheObject)){
            throw new RuntimeException("已发起抽奖，请勿重复操作");
        }
        MarketDrawActivity drawActivity = marketDrawActivityService.getDrawActivityById(drawVo.getDrawActivityId());
        //获取用户抽奖次数
        MarketUserDraw userDrawNumber = marketUserDrawService.getUserDrawNumber(drawVo.getUserId(), drawVo.getDrawActivityId());
        if(userDrawNumber.getAwardNumber() <= 0){
            throw new RuntimeException("抽奖次数已用完");
        }
        Integer winLottery = drawItem(drawActivity.getProbabilitys());
        //结果存储redis中  30秒 出结果后 回调校验
        redisCache.setCacheObject(REDIS_DRAW_ACTIVITY_KEY + drawVo.getDrawActivityId() + drawVo.getUserId(), winLottery, 30, TimeUnit.SECONDS);
        return AjaxResult.success(winLottery);
    }

    public static Integer drawItem(List<MarketDrawActivityProbability> probabilities) {
        Random random = new Random();
        double randomValue = random.nextDouble(); // 0.0到1.0之间的随机数
        double cumulativeProbability = 0.0;

        for (MarketDrawActivityProbability item : probabilities) {
            cumulativeProbability += item.getProbability().doubleValue() / 100;
            if (randomValue <= cumulativeProbability) {
                return item.getPos();
            }
        }
        return drawItem(probabilities); //如果概率总和是1.0
    }

}
