package com.ruoyi.web.controller.callback;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.idgen.IdGenerate;
import com.ruoyi.market.constants.AdvertisingConstants;
import com.ruoyi.market.constants.DrawConstants;
import com.ruoyi.market.domain.MarketAdvertisingLog;
import com.ruoyi.market.domain.MarketUserDraw;
import com.ruoyi.market.domain.vo.AdvertisingBackQueryVo;
import com.ruoyi.market.domain.vo.DrawCallBackQueryVo;
import com.ruoyi.market.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.List;
import java.util.Objects;

@Api(tags = "前端回调管理")
@RestController
@RequestMapping("/callback")
public class CallBackController{

    @Autowired
    RedisCache redisCache;

    @Autowired
    MarketDrawActivityService marketDrawActivityService;

    @Resource
    private MarketAdvertisingLogService marketAdvertisingLogService;

    @Resource
    private MarketAdvertisingAwardService marketAdvertisingAwardService;

    @Resource
    private MarketAwardLibraryService marketAwardLibraryService;

    @Resource
    private MarketUserDrawService marketUserDrawService;

    @ApiOperation("抽奖回调")
    @PostMapping("drawCallback")
    public AjaxResult drawCallback(@RequestBody DrawCallBackQueryVo drawCallBackVo){
        Object cacheObject = redisCache.getCacheObject(DrawConstants.REDIS_DRAW_ACTIVITY_KEY + drawCallBackVo.getDrawActivityId()  + drawCallBackVo.getUserId());
        //找不到redis数据 处理
        if(Objects.isNull(cacheObject)){
            redisCache.deleteObject(DrawConstants.REDIS_DRAW_ACTIVITY_KEY + drawCallBackVo.getDrawActivityId() + drawCallBackVo.getUserId());
            throw new RuntimeException("请求超时");
        }
        if(!drawCallBackVo.getResult().toString().equals(cacheObject.toString())){
            redisCache.deleteObject(DrawConstants.REDIS_DRAW_ACTIVITY_KEY + drawCallBackVo.getDrawActivityId() + drawCallBackVo.getUserId());
            throw new RuntimeException("数据异常");
        }

        //清空redis数据
        redisCache.deleteObject(DrawConstants.REDIS_DRAW_ACTIVITY_KEY + drawCallBackVo.getDrawActivityId() + drawCallBackVo.getUserId());
        //回调奖品发放
        marketDrawActivityService.userDrawOver(drawCallBackVo);
        MarketUserDraw userDrawNumber = marketUserDrawService.getUserDrawNumber(drawCallBackVo.getUserId(), drawCallBackVo.getDrawActivityId());
        return AjaxResult.success("抽奖完成",userDrawNumber.getAwardNumber());
    }

    @ApiOperation("广告回调")
    @PostMapping("advertisingCallback")
    public AjaxResult advertisingCallback(@RequestBody AdvertisingBackQueryVo advertisingBackQueryVo) {
        // 回调生成用户观看广告记录
        MarketAdvertisingLog marketAdvertisingLog = new MarketAdvertisingLog();
        BeanUtils.copyProperties(advertisingBackQueryVo, marketAdvertisingLog);
        marketAdvertisingLog.setId(IdGenerate.nextId());
        marketAdvertisingLog.setCreateTime(LocalDateTime.now());
        marketAdvertisingLogService.save(marketAdvertisingLog);
        Object cacheObject = redisCache.getCacheObject(AdvertisingConstants.REDIS_ADVERTISING_KEY + advertisingBackQueryVo.getUserId());
        //找不到redis数据 处理
        if(Objects.isNull(cacheObject)){
            redisCache.deleteObject(AdvertisingConstants.REDIS_ADVERTISING_KEY + advertisingBackQueryVo.getUserId());
            throw new RuntimeException("请求超时");
        }
        if(!advertisingBackQueryVo.getVideoId().equals(cacheObject.toString())){
            redisCache.deleteObject(AdvertisingConstants.REDIS_ADVERTISING_KEY + advertisingBackQueryVo.getUserId());
            throw new RuntimeException("数据异常");
        }
        redisCache.deleteObject(AdvertisingConstants.REDIS_ADVERTISING_KEY + advertisingBackQueryVo.getUserId());
        //暂时。。。。。
        if (advertisingBackQueryVo.getIsCatOver().equals("1")) {
            marketAwardLibraryService.adsGrantAward(advertisingBackQueryVo.getVideoId(), advertisingBackQueryVo.getUserId());
            return AjaxResult.success("奖励已发放");
        }
        return AjaxResult.success("广告已播放");
    }

    @ApiOperation("观看广告日志")
    @PostMapping("advertisingLog")
    public AjaxResult advertisingLog(@RequestBody MarketAdvertisingLog marketAdvertisingLog) {
        MarketAdvertisingLog marketAdvertisingLogs = new MarketAdvertisingLog();
        marketAdvertisingLogs.setId(IdGenerate.nextId());
        marketAdvertisingLogs.setUserId(marketAdvertisingLog.getUserId());
        marketAdvertisingLogs.setIsCatOver("0");
        marketAdvertisingLogs.setVideoId("");
        marketAdvertisingLogs.setCreateTime(LocalDateTime.now());
        marketAdvertisingLogService.save(marketAdvertisingLogs);
        return AjaxResult.success("",marketAdvertisingLogs.getId());
    }

    @ApiOperation("验证是否观看完成")
    @GetMapping("verifyWatch")
    public AjaxResult verifyWatch(@RequestParam("id") String id) {
        MarketAdvertisingLog marketAdvertisingLogs = marketAdvertisingLogService.getById(id);
        if (marketAdvertisingLogs != null) {
            String videoId = marketAdvertisingLogs.getVideoId();
            String isCatOver = marketAdvertisingLogs.getIsCatOver();
            if (!videoId.equals("") && isCatOver.equals("1")){
                return AjaxResult.success("观看完成", true);
            }
        }
        return AjaxResult.success("观看未完成", false);
    }


    @ApiOperation("广告回调")
    @GetMapping("testAdvertisingCallback")
    public AjaxResult testAdvertisingCallback(
            @RequestParam("adpid") String adpid,
            @RequestParam("provider") String provider,
            @RequestParam("platform") String platform,
            @RequestParam("sign") String sign,
            @RequestParam("trans_id") String transId,
            @RequestParam("user_id") String userId,
            @RequestParam("extra") String extra) {
        // 验证签名
        if (!validateSignature(sign, transId)) {
            return AjaxResult.error("无效的签名");
        }
        JSONObject obj = JSON.parseObject(extra);
        String id = obj.getString("id");
        System.out.println("id:" + extra);
        MarketAdvertisingLog marketAdvertisingLog = marketAdvertisingLogService.getById(id);
        if (marketAdvertisingLog != null) {
            marketAdvertisingLog.setIsCatOver("1");
            marketAdvertisingLog.setVideoId(transId);
            marketAdvertisingLogService.updateById(marketAdvertisingLog);
        } else {
            // 回调生成用户观看广告记录
            MarketAdvertisingLog newMarketAdvertisingLog = new MarketAdvertisingLog();
            newMarketAdvertisingLog.setId(IdGenerate.nextId());
            newMarketAdvertisingLog.setVideoId(transId);
            newMarketAdvertisingLog.setIsCatOver("1");
            newMarketAdvertisingLog.setUserId(userId);
            newMarketAdvertisingLog.setCreateTime(LocalDateTime.now());
            marketAdvertisingLogService.save(newMarketAdvertisingLog);
        }
        //获取该用户当天观看广告次数
        LocalDateTime startOfDay = LocalDateTime.of(LocalDate.now(), LocalTime.MIN);
        LocalDateTime endOfDay = LocalDateTime.of(LocalDate.now(), LocalTime.MAX);
        LambdaQueryWrapper<MarketAdvertisingLog> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(MarketAdvertisingLog::getUserId, userId)
                .eq(MarketAdvertisingLog::getIsCatOver, 1)
                .between(MarketAdvertisingLog::getCreateTime, startOfDay, endOfDay);
        String watchCount = String.valueOf(marketAdvertisingLogService.count(queryWrapper));

        //用户大转盘抽奖次数+1
        if (watchCount.equals("5")) {
            MarketUserDraw marketUserDraw = marketUserDrawService.getUserDrawNumber(userId, "1824703218157920256");
            if (marketUserDraw == null) {
                MarketUserDraw newMarketUserDraw = new MarketUserDraw();
                newMarketUserDraw.setId(IdGenerate.nextId());
                newMarketUserDraw.setActivityId("1824703218157920256");
                newMarketUserDraw.setUserId(userId);
                newMarketUserDraw.setAwardNumber(1);
                marketUserDrawService.save(newMarketUserDraw);
            } else {
                marketUserDraw.setAwardNumber(marketUserDraw.getAwardNumber() + 1);
                marketUserDrawService.updateById(marketUserDraw);
            }
            return AjaxResult.success("广告观看完成，奖励已发放");
        }
        return AjaxResult.success("广告观看完成");
    }

    private boolean validateSignature(String providedSign, String transId) {
        // 生成签名
        String calculatedSign = generateSignature(transId);
        // 验证签名是否匹配
        return providedSign.equals(calculatedSign);
    }

    private String generateSignature(String transId) {
        // 使用固定的密钥或动态生成的密钥
        String secretKey = "4e00ad721e1915dcadbe5888af1201565733c5d5da01f01fb63fc011c700afa4";

        // 构造待签名的字符串
        String toSign = secretKey + ":" + transId;

        // 使用SHA-256算法生成签名
        try {
            MessageDigest digest = MessageDigest.getInstance("SHA-256");
            byte[] hash = digest.digest(toSign.getBytes());
            StringBuilder hexString = new StringBuilder();
            for (byte b : hash) {
                String hex = Integer.toHexString(0xff & b);
                if (hex.length() == 1) hexString.append('0');
                hexString.append(hex);
            }
            return hexString.toString();
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException("Error generating signature: " + e.getMessage(), e);
        }
    }
}
