package com.ruoyi.web.controller.market;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.annotation.RequestSingleParam;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.idgen.IdGenerate;
import com.ruoyi.market.domain.MarketAwardLibrary;
import com.ruoyi.market.domain.vo.AwardQueryVo;
import com.ruoyi.market.service.MarketAwardLibraryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * 奖品 controller
 */
@Api(tags = "奖品管理")
@RestController
@RequestMapping("/market/award")
public class AwardController {
    @Autowired
    private MarketAwardLibraryService marketAwardLibraryService;

    /**
     * 获取奖品列表
     * @return
     */
    @ApiOperation("获取奖品列表")
    @PostMapping("getAwardList")
    public AjaxResult  getAwardList(@RequestBody AwardQueryVo awardQueryVo){
        Page page = awardQueryVo.getPage();
        return AjaxResult.success(marketAwardLibraryService.awardLibraryList(page,awardQueryVo));
    }

    /**
     * 添加奖品
     * @param marketAwardLibrary
     * @return
     */
    @ApiOperation("添加奖品")
    @PostMapping("addAward")
    public AjaxResult addAward(@Validated @RequestBody MarketAwardLibrary marketAwardLibrary){
        marketAwardLibrary.setId(IdGenerate.nextId());
        return AjaxResult.success(marketAwardLibraryService.save(marketAwardLibrary));
    }

    /**
     * 删除奖品
     * @param awardId
     * @return
     */
    @ApiOperation("删除奖品")
    @PostMapping("delAward")
    public AjaxResult delAward(@RequestSingleParam(value = "id")String awardId){
        return marketAwardLibraryService.delAward(awardId) ? AjaxResult.success("删除奖品成功") : AjaxResult.error("删除奖品失败");
    }

    @ApiOperation("修改商品")
    @PostMapping("updateAward")
    public AjaxResult updateAward(@Validated @RequestBody MarketAwardLibrary marketAwardLibrary){
        return marketAwardLibraryService.updateById(marketAwardLibrary) ? AjaxResult.success("修改奖品成功") : AjaxResult.error("修改奖品失败");
    }


}
