package com.ruoyi.web.controller.market;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.market.domain.MarketAwardLibrary;
import com.ruoyi.market.domain.MarketAwardType;
import com.ruoyi.market.domain.vo.AwardTypeVo;
import com.ruoyi.market.service.MarketAwardLibraryService;
import com.ruoyi.market.service.MarketAwardTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Objects;

/**
 * 奖品 controller
 */
@Api(tags = "奖品类型管理")
@RestController
@RequestMapping("/market/awardType")
public class AwardTypeController {
    @Autowired
    private MarketAwardTypeService marketAwardTypeService;

    @Autowired
    private MarketAwardLibraryService marketAwardLibraryService;

    /**
     * 获取奖品列表
     * @return
     */
    @ApiOperation("获取奖品类型列表")
    @PostMapping("getAwardTypeList")
    public AjaxResult getAwardTypeList(@RequestBody AwardTypeVo awardTypeVo){
        Page page = awardTypeVo.getPage();
        LambdaQueryWrapper<MarketAwardType> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(!Objects.isNull(awardTypeVo.getName()), MarketAwardType::getTypeName, awardTypeVo.getName());
        Page<MarketAwardType> resultPage = marketAwardTypeService.page(page, queryWrapper);
        return AjaxResult.success(resultPage);
    }



    /**
     * 添加奖品类型
     * @param marketAwardType
     * @return
     */
    @ApiOperation("添加奖品类型")
    @PostMapping("addAwardType")
    public AjaxResult addAwardType(@RequestBody MarketAwardType marketAwardType){
        return AjaxResult.success(marketAwardTypeService.save(marketAwardType));
    }

    /**
     * 删除奖品类型
     * @param marketAwardType
     * @return
     */
    @ApiOperation("删除奖品类型")
    @PostMapping("delAwardType")
    public AjaxResult delAwardType(@RequestBody MarketAwardType marketAwardType){
        List<MarketAwardLibrary> awardLibraryList = marketAwardLibraryService
                .list(new LambdaQueryWrapper<MarketAwardLibrary>().eq(MarketAwardLibrary::getAwardType, marketAwardType.getId()));
        if (!awardLibraryList.isEmpty()) {
            return AjaxResult.error("该奖品类型下有奖品，无法删除奖品类型");
        }
        return marketAwardTypeService.removeById(marketAwardType.getId()) ? AjaxResult.success("删除奖品类型成功") : AjaxResult.error("删除奖品类型失败");
    }

    @ApiOperation("修改奖品类型")
    @PostMapping("updateAward")
    public AjaxResult updateAward(@RequestBody MarketAwardType marketAwardType){
        return marketAwardTypeService.updateById(marketAwardType) ? AjaxResult.success("修改奖品类型成功") : AjaxResult.error("修改奖品类型失败");
    }


}
