package com.ruoyi.web.controller.market;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.annotation.RequestSingleParam;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.market.domain.MarketDrawTemplate;
import com.ruoyi.market.domain.vo.DrawTemplateQueryVo;
import com.ruoyi.market.service.MarketDrawTemplateService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;


/**
 * 抽奖 controller
 */
@Api(tags = "抽奖模版管理")
@RestController
@RequestMapping("/market/drawTemplate")
public class DrawTemplateController {

    @Autowired
    private MarketDrawTemplateService marketDrawTemplateService;

    /**
     * 获取模版抽奖list
     * @return
     */
    @ApiOperation("获取抽奖模板列表")
    @PostMapping(value = "getDrawTemplateList")
    public AjaxResult getDrawTemplateList(@RequestBody DrawTemplateQueryVo awardTemplateQueryVo) {
        Page page = awardTemplateQueryVo.getPage();
        return AjaxResult.success(marketDrawTemplateService.getDrawTemplatePage(page,awardTemplateQueryVo));
    }

    /**
     * 添加模版抽奖
     * @param marketDrawTemplate
     * @return
     */
    @ApiOperation("添加模板抽奖")
    @PostMapping(value = "addDrawTemplate")
    public AjaxResult addDrawTemplate(@Validated @RequestBody MarketDrawTemplate marketDrawTemplate) {
        marketDrawTemplate.setCreateTime(LocalDateTime.now());
        return AjaxResult.success(marketDrawTemplateService.save(marketDrawTemplate));
    }

    /**
     * 更新抽奖模版
     * @param marketDrawTemplate
     * @return
     */
    @ApiOperation("更新活动抽奖模板")
    @PostMapping(value = "updateDrawTemplate")
    public AjaxResult updateDrawTemplate(@Validated @RequestBody MarketDrawTemplate marketDrawTemplate) {
        return AjaxResult.success(marketDrawTemplateService.updateById(marketDrawTemplate));
    }

    @ApiOperation("删除抽奖模版")
    @PostMapping(value = "delDrawTemplate")
    public AjaxResult delDrawTemplate(@RequestSingleParam(value = "drawTemplateId") String drawTemplateId) {
        return AjaxResult.success(marketDrawTemplateService.delDrawTemplate(drawTemplateId));
    }

}
