package com.ruoyi.web.controller.market;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.PageDomain;
import com.ruoyi.market.domain.MarketSystem;
import com.ruoyi.market.domain.MarketUserWechat;
import com.ruoyi.market.service.MarketSystemService;
import com.ruoyi.market.service.MarketUserWechatService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

@Api(tags = "系统管理")
@RestController
@RequestMapping("/market/system")
public class MarketSystemController {

    @Resource
    private MarketSystemService marketSystemService;

    @Resource
    private MarketUserWechatService marketUserWechatService;

    @ApiOperation("获取系统列表")
    @PostMapping("getSystemList")
    public AjaxResult getSystemList(@RequestBody PageDomain pageDomain){
        Page page = pageDomain.getPage();
        return AjaxResult.success(marketSystemService.page(page));
    }

    @ApiOperation("添加系统")
    @PostMapping("addSystem")
    public AjaxResult addSystem(@RequestBody MarketSystem marketSystem){
        return AjaxResult.success(marketSystemService.insertMarketSystem(marketSystem));
    }

    @ApiOperation("修改系统")
    @PostMapping("updateSystem")
    public AjaxResult updateSystem(@RequestBody MarketSystem marketSystem){
        return marketSystemService.updateMarketSystem(marketSystem) ? AjaxResult.success("修改成功") : AjaxResult.error("修改失败");
    }


    @ApiOperation("删除系统")
    @PostMapping("delSystem")
    public AjaxResult delSystem(@RequestBody MarketSystem marketSystem){
        MarketSystem system = marketSystemService.getById(marketSystem.getId());
        List<MarketUserWechat> userWechatList = marketUserWechatService.list(new LambdaQueryWrapper<MarketUserWechat>().eq(MarketUserWechat::getSystemCode, system.getSystemCode()));
        if (!userWechatList.isEmpty()) {
            return AjaxResult.error("该系统下有用户绑定,无法删除");
        }
        return marketSystemService.removeById(marketSystem.getId()) ? AjaxResult.success("删除该系统成功") : AjaxResult.error("删除该系统失败");
    }
}
