package com.ruoyi.web.controller.wechat;

import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.market.domain.MarketDrawActivity;
import com.ruoyi.market.domain.MarketDrawTemplate;
import com.ruoyi.market.domain.MarketUserWechat;
import com.ruoyi.market.domain.vo.WechatVo;
import com.ruoyi.market.service.MarketDrawActivityService;
import com.ruoyi.market.service.MarketDrawTemplateService;
import com.ruoyi.market.service.MarketUserWechatService;
import com.ruoyi.market.service.WechatPayService;
import com.wechat.pay.java.service.transferbatch.model.InitiateBatchTransferResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.Base64;
import java.util.concurrent.TimeUnit;

@Api(tags = "微信管理")
@RestController
@RequestMapping("/wechat")
public class WechatController {
    @Resource
    private RedisCache redisCache;

    @Resource
    private MarketDrawActivityService marketDrawActivityService;

    @Resource
    private MarketDrawTemplateService marketDrawTemplateService;

    @Resource
    private MarketUserWechatService marketUserWechatService;

    @Resource
    private WechatPayService wechatPayService;

    private static final String WECHAT_ACCESS_TOKEN_KEY = "access_token";

    private static final String APPID = "wx4014825e79c10552";

    private static final String APPSECRET = "7adf04002c80ca20e7d29aec301ee00e";
    public String fetchAccessToken() {
        String accessTokenUrl = String.format("https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=%s&secret=%s", APPID, APPSECRET);
        String response = HttpUtil.get(accessTokenUrl);
        JSONObject jsonObject = JSON.parseObject(response);
        return jsonObject.getString("access_token");

    }

    public String getAccessToken() {
        String accessToken = redisCache.getCacheObject(WECHAT_ACCESS_TOKEN_KEY);

        if (StringUtils.isEmpty(accessToken)) {
            accessToken = fetchAccessToken();
            // access_token 有效期为 7200 秒
            redisCache.setCacheObject(WECHAT_ACCESS_TOKEN_KEY, accessToken, 7000, TimeUnit.SECONDS);
        }
        return accessToken;
    }

    @PostMapping("/getQrCode")
    @ApiOperation("获取小程序码")
    public String getQrCode(@RequestBody WechatVo wechatVo) {
        String url = "https://api.weixin.qq.com/cgi-bin/wxaapp/createwxaqrcode?access_token=" + getAccessToken();

        HttpClient httpClient = HttpClients.createDefault();
        HttpPost httpPost = new HttpPost(url);

        // 构建请求参数，这里示意为一个 JSON 格式的请求体
        JSONObject params = new JSONObject();
        String sceneValue = wechatVo.getUserId() +' '+ wechatVo.getActivityId();
        MarketDrawActivity activity = marketDrawActivityService.getById(wechatVo.getActivityId());
        MarketDrawTemplate template = marketDrawTemplateService.getById(activity.getTemplateId());
        params.put("path", template.getTemplateAppletPage() + "?userId=" + wechatVo.getUserId() + "&activityId=" + wechatVo.getActivityId()); // 路径
        params.put("env_version", "trial");
        try {
            // 验证用户信息是否存在
            MarketUserWechat userWechat = marketUserWechatService.getById(wechatVo.getUserId());
            if (userWechat == null) {

            }

            StringEntity entity = new StringEntity(params.toJSONString(), "UTF-8");
            httpPost.setEntity(entity);
            httpPost.setHeader("Content-Type", "application/json");

            HttpResponse response = httpClient.execute(httpPost);
            HttpEntity responseEntity = response.getEntity();

            if (responseEntity != null) {
                byte[] bytes = EntityUtils.toByteArray(responseEntity);
                // 返回 Base64 编码的字符串
                return Base64.getEncoder().encodeToString(bytes);
            }
        } catch (Exception e) {
            e.printStackTrace();
            // 处理异常情况
            return "Failed: " + e.getMessage();
        }

        return "Failed: Unknown error";
    }

    @PostMapping("/getUserInfo")
    @ApiOperation("获取用户信息")
    public AjaxResult getUserInfo(@RequestBody WechatVo wechatVo) throws IOException {
        String jsCode = wechatVo.getCode();
        String url = String.format("https://api.weixin.qq.com/sns/jscode2session?appid=%s&secret=%s&js_code=%s&grant_type=authorization_code", APPID, APPSECRET, jsCode);

        CloseableHttpClient httpClient = HttpClients.createDefault();
        HttpGet httpGet = new HttpGet(url);

        CloseableHttpResponse response = httpClient.execute(httpGet);
        try {
            // 获取响应实体
            HttpEntity entity = response.getEntity();
            if (entity != null) {
                // 将响应实体转换为字符串
                String result = EntityUtils.toString(entity);
                JSONObject jsonObject = JSON.parseObject(result);
                if (jsonObject != null) {
                    if (jsonObject.containsKey("openid")) {
                        String openid = jsonObject.getString("openid");
                        // 验证用户信息是否存在
                        MarketUserWechat userWechat = marketUserWechatService.getOne(new LambdaQueryWrapper<MarketUserWechat>()
                                .eq(MarketUserWechat::getOpenId, openid));
                        if (userWechat == null) {
                            userWechat = new MarketUserWechat();
                            userWechat.setUserId(wechatVo.getUserId());
                            userWechat.setOpenId(openid);
                            marketUserWechatService.save(userWechat);
                            return AjaxResult.success("获取用户信息成功", userWechat);
                        } else if (!userWechat.getUserId().equals(wechatVo.getUserId())) {
                            return AjaxResult.error("该微信账户已绑定其他用户,请联系管理员");
                        }
                    }

                } else {
                    System.err.println("Failed to get openid from response");
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return AjaxResult.error("获取用户信息失败");
    }

    @GetMapping("/transfer")
    public InitiateBatchTransferResponse transfer(@RequestParam("openId") String openId, @RequestParam("amount") Long amount) throws IOException {
        return wechatPayService.transfer(openId, amount);
    }
}
